import click

from jsonfeed2social.config import init_config
from jsonfeed2social.jsonfeed import ManagerFeed
from jsonfeed2social.social.manager import sender


@click.group()
def cli():
    pass


@cli.command()
@click.option("--config", default="config.ini", help="Path to the config file")
@click.option("--populatecache", default=False, help="Populate the cache")
def run(config, populatecache):
    config = init_config(config)
    sections = config.sections()
    if all(["twitter" not in sections, "mastodon" not in sections]):
        click.echo(
            "You need to set the twitter config or the mastodon config for working"
        )
    sender(config, populatecache)


@cli.command()
@click.option("--config", default="config.ini", help="Path to the config file")
@click.option(
    "--consumer_key",
    prompt="Yout twitter consumer key",
    help="Yout twitter consumer key",
)
@click.option(
    "--consumer_secret",
    prompt="Your twitter consumer secret",
    help="Your twitter consumer secret",
)
@click.option(
    "--access_token",
    prompt="Your twitter access token",
    help="Your twitter access token",
)
@click.option(
    "--access_token_secret",
    prompt="Your twitter token secret",
    help="Your twitter token secret",
)
def twitter(
    config, consumer_key, consumer_secret, access_token, access_token_secret
):
    config_path = config
    config = init_config(config_path)
    config["twitter"] = {
        "twitter_consumer_key": consumer_key,
        "twitter_consumer_secret": consumer_secret,
        "twitter_access_token": access_token,
        "twitter_access_token_secret": access_token_secret,
    }
    with open(config_path, "w") as configfile:
        config.write(configfile)


@cli.command("feed-sections")
@click.option("--config", default="config.ini", help="Path to the config file")
def feed_sections(config):
    config_path = config
    config = init_config(config_path)
    feed = ManagerFeed(config["feed"]["uri"])
    data = feed.feed_sections
    click.echo(f"The following sections are available in this feed {data}")



from configparser import ConfigParser

import tweepy

from jsonfeed2social.utility import get_message


def send_tweet(
    status: str,
    api_key: str,
    api_secrets: str,
    access_token: str,
    access_secret: str,
):
    # Authenticate to Twitter
    auth = tweepy.OAuthHandler(api_key, api_secrets)
    auth.set_access_token(access_token, access_secret)
    api = tweepy.API(auth, wait_on_rate_limit=True)
    api.update_status(status=status)


def twitter_sender(config: ConfigParser, news: list):
    for e in news:
        send_tweet(
            get_message(e, config["feed"]["tweet"]),
            config["twitter"]["twitter_consumer_key"],
            config["twitter"]["twitter_consumer_secret"],
            config["twitter"]["twitter_access_token"],
            config["twitter"]["twitter_access_token_secret"],
        )



import re

from jinja2 import Template
from jinja2 import Environment, meta

from jsonfeed2social.exception import FildNotInData


def get_message(data_old: dict, message_format: str):
    data = data_old
    env = Environment()
    ast = env.parse(message_format)
    for e in meta.find_undeclared_variables(ast):
        if not data.get(e, False) and e != "tags":
            raise FildNotInData(f"You need {e} for the message")
    if "tags" in data.keys():
        tags = ""
        for e in data["tags"]:
            tag = re.sub(r"[^a-zA-Z0-9]", "", e)
            tags += f"#{tag} "
        data["tags"] = tags.strip()
    rtemplate = Template(message_format)
    data = rtemplate.render(**data)
    return data



import configparser

import click


def init_config(path_config: str):
    config = configparser.ConfigParser()
    config["feed"] = {}
    config["cache"] = {
        "cachefile": "config/json2social.db",
        "cache_limit": "10000",
    }
    config.read(path_config)
    try:
        config["feed"]["uri"]
    except KeyError:
        config["feed"]["uri"] = input("The URI of the feed\n")
    with open(path_config, "w") as configfile:
        config.write(configfile)
    sections = config.sections()
    if all(["twitter" not in sections, "mastodon" not in sections]):
        click.echo(
            "You need to set the twitter config or the mastodon config for working"
        )
    return config
